// Master I2C implementation
// Master functions are done bit-wise

BOOLEAN DataOutBit;
BOOLEAN DI;

#bit		SDAEn=TRISC.5
#bit		SCLEn=TRISC.0
#bit		SDA=PortC.5
#bit		SCL=PortC.0

Master_Init()
{
	SCL = 0;				// Set SDA, SCL low when not in tri-state
	SDA = 0;				// Set SDA, SCL low when not in tri-state
	SCLEn = 1;			// SCL and SDA are inputs
	SDAEn = 1;			// SCL and SDA are inputs
}

StartM()
{
	SDAEn = 1;			//
	SCLEn = 1;
	SDA = 0;				// Set SDA, SCL low when not in tri-state
	SDAEn = 0; 			// SDA goes low during SCL high
	delay_us(5);		// Timing adjustment, .6uS Fast mode, 4.7 standard mode
	SCL = 0;				// Set SDA, SCL low when not in tri-state
	SCLEn = 0;			// Start clock train
}

StopM()
{
	SDA = 0;				// Set SDA, SCL low when not in tri-state
	SDAEn = 0;			// Return SDA to low
	delay_cycles(2);
	SCLEn = 1;			// Set SCL high
	delay_us(5);
	SDAEn = 1;			// SDA goes from low to high during SCL high
}

BitOut()
{
	int count;

	if (DataOutBit)
		SDAEn = 1;			 		// Output bit 1
	else
		SDAEn = 0;					// Output bit 0

	delay_cycles(2);
	SCLEn = 1; 						// Allow SCL to go high
	delay_us(5);
	SCLEn = 0;  					// return Clock to low state
}

BitIn()
{
	SDAEn = 1;     	// Set SDA for input
	DI = 0;    			// may also function as a delay!
	SCLEn = 1;			// Clock high
	delay_us(3);

	if(SDA) 				// Read SDA pin
		DI = 1; 			// for our returned data else leave it cleared
	delay_us(2);

	SCLEn = 0; 			// Return SCL to low
}

void XmitM(int OutData)
{
	int Count;

	for(Count = 0; Count < 8; Count++)
	{
		SDAEn = shift_left(&OutData,1,0);
		delay_cycles(2);
		SCLEn = 1; 						// Allow SCL to go high
		delay_us(5);
		SCLEn = 0;  					// return Clock to low state
	}

	SDAEn = 1;     					// Set SDA for input
	delay_cycles(2);
	SCLEn = 1;							// Clock high
	delay_us(5);
	SCLEn = 0; 							// Return SCL to low
}

void ReadM(int &Received)
{
	int Count = 8;

	Received = 0;

	while(Count > 0)
	{
		Count--;
		BitIn();
		if(DI)
			Received |= bits[Count];
	}

	DataOutBit = 1; // Set acknowledge bit = 1 to STOP further input
	BitOut();
}
